// data structures for only the receive FPGA
//


#ifndef _RXFPGA_H
#define _RXFPGA_H



#include "../apptel/xfpga.h"



struct RxControl1_register
{
    // active low
    //
    // generates a reset to external FIFO's, which causes them
    // to set their read and write pointers to location zero
    //
    // defaults to active on powerup, so startup code must
    // release it to allow cells to be written to FIFO
    //
    Bit8 UnResetExternalFifo    : 1;

    // enables cells to be read from S/UNI-622
    //
    Bit8 EnableCells            : 1;

    Bit8 EnableFifoWrite        : 1;

    // Enables external FIFO's to write to FIFO interface
    // of AMCC S5933Q "matchmaker" PCI bus interface
    //
    Bit8 EnableAmccWrite        : 1;

    // active low
    //
    Bit8 UnLoadFifoLevels       : 1;

    Bit8 CopyTimestampsToHost   : 1;

    Bit8 CopyAtmHeadersToHost   : 1;

    Bit8 CopyAtmPayloadsToHost  : 1;
};



struct RxStatus1_register
{
    // Read only
    //
    Bit8 FifoEmpty              : 1;

    // Read only
    //
    // default setup of external FIFO's means this signal
    // will be asserted when they have 8 or fewer words
    // filled
    //
    Bit8 FifoAlmostEmpty        : 1;

    // Read only
    //
    // default setup of external FIFO's means this signal
    // will be asserted when they have 8 or fewer words
    // available
    //
    Bit8 FifoAlmostFull        : 1;

    // Read only
    //
    // note: when the external FIFO gets full, the RX FPGA
    // discards data until the FIFO is not full
    //
    Bit8 FifoFull               : 1;

    // read only, cleared upon read
    //
    // has FifoAlmostFull field above ever been TRUE?
    //
    Bit8 FifoEverAlmostFull     : 1;

    // read only, cleared upon read
    //
    // has FifoFull field above ever been TRUE?
    //
    Bit8 FifoEverFull           : 1;

    // read only, cleared upon read
    //
    // Has the 16-bit data from the AIM622
    // ever not matched its 2-bit parity?
    //
    Bit8 AimEverRxParityError   : 1;

    Bit8 Reserved               : 1;
};



struct RxFifoLevelWrite_register
{
    Bit8 Value;
};



struct RxControl2_register
{
    // 0  means only VPI[7:0] and VCI[15:0] used for state indexing
    // 1  means VPI8  (GFC0) replaces VCI15
    // 2  means VPI9  (GFC1) replaces VCI14
    // 3  means VPI10 (GFC2) replaces VCI13
    // 4  means VPI12 (GFC3) replaces VCI12
    // 5-7 mean same as 0
    //
    Bit8 NumVciBitsStolenForVpi : 3;

    Bit8 PassAllNonLastCells    : 1;

    Bit8 PassOamRmCells         : 1;

    Bit8 PassAllLastCells       : 1;

    // valid range: 0 to 3
    //
    Bit8 NumNonLastCellsToPass  : 2;
};



struct RxMatchUnusedBits_register
{
    // bit 0 matched against VPI8  (or VCI15 if 0<NumVciBitsStolenForVpi<5)
    // bit 1 matched against VPI9  (or VCI14 if 1<NumVciBitsStolenForVpi<5)
    // bit 2 matched against VPI10 (or VCI13 if 2<NumVciBitsStolenForVpi<5)
    // bit 3 matched against VPI11 (or VCI12 if 3<NumVciBitsStolenForVpi<5)
    //
    Bit8 Value                  : 4;

    Bit8 Mask                   : 4;
};



// holds bits 7:0 of address presented to SRAM for host
//
struct RxSramAddress0_register
{
    Bit8 Value;
};



// holds bits 15:8 of address presented to SRAM for host
//
struct RxSramAddress1_register
{
    Bit8 Value;
};



struct RxSramAddress2Control_register
{
    // holds bits 19:16 of address presented to SRAM for host
    //
    Bit8 Value                  : 4;

    Bit8 Reserved               : 2;

    Bit8 CycleIsWrite           : 1;

    // set by host to initiate an SRAM cycle
    // cleared by FPGA when cycle is done
    //
    Bit8 DoCycle                : 1;
};



#define MAX_SRAM_ADDRESS        ((1 << 20) - 1)



// holds bits 7:0 of data that was last read from
// or will next be written to SRAM for host
//
struct RxSramData0_register
{
    Bit8 Value;
};



// holds bits 15:8 of data that was last read from
// or will next be written to SRAM for host
//
struct RxSramData1_register
{
    Bit8 Value;
};



// holds bits 23:16 of data that was last read from
// or will next be written to SRAM for host
//
struct RxSramData2_register
{
    Bit8 Value;
};



// holds bits 31:24 of data that was last read from
// or will next be written to SRAM for host
//
struct RxSramData3_register
{
    Bit8 Value;
};



struct RxCellCountTransferReset_register
{
    // set by host to initiate the transfer and clearing
    // cleared by FPGA when operation is done
    //
    Bit8 LatchAndClear          : 1;

    Bit8 Reserved               : 7;
};



// hold bits 7:0 of latched count of OAM and RM cells
//
struct RxOamRmCellCountRead0_register
{
    Bit8 Value;
};



// hold bits 15:8 of latched count of OAM and RM cells
//
struct RxOamRmCellCountRead1_register
{
    Bit8 Value;
};



// hold bits 23:16 of latched count of OAM and RM cells
//
struct RxOamRmCellCountRead2_register
{
    Bit8 Value;
};



// hold bits 31:24 of latched count of OAM and RM cells
//
struct RxOamRmCellCountRead3_register
{
    Bit8 Value;
};



// hold bits 7:0 of latched count of mismatched NNI cells
//
struct RxNniMismatchCountRead0_register
{
    Bit8 Value;
};



// hold bits 15:8 of latched count of mismatched NNI cells
//
struct RxNniMismatchCountRead1_register
{
    Bit8 Value;
};



// hold bits 23:16 of latched count of mismatched NNI cells
//
struct RxNniMismatchCountRead2_register
{
    Bit8 Value;
};



// hold bits 31:24 of latched count of mismatched NNI cells
//
struct RxNniMismatchCountRead3_register
{
    Bit8 Value;
};



// merged registers
//
struct RxFpga_registers
{
    union
    {
        RxControl1_register Control1;
        Bit32 dword_0;
    };

    union
    {
        RxStatus1_register Status1;
        Bit32 dword_1;
    };

    union
    {
        RxFifoLevelWrite_register FifoLevelWrite;
        Bit32 dword_2;
    };

    union
    {
        RxControl2_register Control2;
        Bit32 dword_3;
    };

    union
    {
        RxMatchUnusedBits_register MatchUnusedBits;
        Bit32 dword_4;
    };

    union
    {
        RxSramAddress0_register SramAddress0;
        Bit32 dword_5;
    };

    union
    {
        RxSramAddress1_register SramAddress1;
        Bit32 dword_6;
    };

    union
    {
        RxSramAddress2Control_register SramAddress2Control;
        Bit32 dword_7;
    };

    union
    {
        RxSramData0_register SramData0;
        Bit32 dword_8;
    };

    union
    {
        RxSramData1_register SramData1;
        Bit32 dword_9;
    };

    union
    {
        RxSramData2_register SramData2;
        Bit32 dword_10;
    };

    union
    {
        RxSramData3_register SramData3;
        Bit32 dword_11;
    };

    union
    {
        RxCellCountTransferReset_register CellCountTransferReset;
        Bit32 dword_12;
    };

    union
    {
        RxOamRmCellCountRead0_register OamRmCellCountRead0;
        Bit32 dword_13;
    };

    union
    {
        RxOamRmCellCountRead1_register OamRmCellCountRead1;
        Bit32 dword_14;
    };

    union
    {
        RxOamRmCellCountRead2_register OamRmCellCountRead2;
        Bit32 dword_15;
    };

    union
    {
        RxOamRmCellCountRead3_register OamRmCellCountRead3;
        Bit32 dword_16;
    };

    union
    {
        RxNniMismatchCountRead0_register NniMismatchCountRead0;
        Bit32 dword_17;
    };

    union
    {
        RxNniMismatchCountRead1_register NniMismatchCountRead1;
        Bit32 dword_18;
    };

    union
    {
        RxNniMismatchCountRead2_register NniMismatchCountRead2;
        Bit32 dword_19;
    };

    union
    {
        RxNniMismatchCountRead3_register NniMismatchCountRead3;
        Bit32 dword_20;
    };

    Bit32 Reserved1[ 9 ];

    union
    {
        Diagnostic_register Diagnostic;
        Bit32 dword_30;
    };

    union
    {
        AddressXor_register AddressXor;
        Bit32 dword_31;
    };

    Bit32 Reserved2[ 224 ];
};



#endif      // not included yet
