#ifndef _TRACE_H
#define _TRACE_H

#include "..\intel\timestmp.h"          // class PentiumClock
#include "..\aal5\atm.h"                // struct AtmHeader
#include "..\wattcp\tcp.h"              // typedef u_long u_short u_char
#include "..\integrat\constant.h"
#include CARD_CLOCK_HEADER              // class CardClock
#include "..\integrat\ip.h"
#include "..\integrat\ip_icmp.h"
#include "..\integrat\igmp.h"
#include "..\integrat\tcp.h"
#include "..\integrat\udp.h"



/*
 * tracerec, tracerec_2 and tracerec_last overlay the raw data
 *
 *
 *
 * these are buildable for 6 possible encapsulations, depending
 * upon how you set up the #define's in CONSTANT.H
 *
 * (1) no encapsulation at all:
 *      HOST_SKIPS_GIGASWITCH       FALSE
 *      HOST_SKIPS_LANE             FALSE
 *      LINK_HAS_LLC_SNAP           FALSE
 *      LINK_HAS_NLPID              FALSE
 *
 * (2) just NLPID encapsulation
 *      HOST_SKIPS_GIGASWITCH       FALSE
 *      HOST_SKIPS_LANE             FALSE
 *      LINK_HAS_LLC_SNAP           FALSE
 *      LINK_HAS_NLPID              TRUE
 *
 * (3) just LLC/SNAP encapsulation
 *      HOST_SKIPS_GIGASWITCH       FALSE
 *      HOST_SKIPS_LANE             FALSE
 *      LINK_HAS_LLC_SNAP           TRUE
 *      LINK_HAS_NLPID              don't care
 *
 * (4) just DIGITAL FDDI GigaSwitch encapsulation
 *      HOST_SKIPS_GIGASWITCH       TRUE
 *      HOST_SKIPS_LANE             FALSE
 *      LINK_HAS_LLC_SNAP           FALSE
 *      LINK_HAS_NLPID              FALSE
 *
 * (5) DIGITAL FDDI GigaSwitch encapsulation followed by LLC/SNAP
 *      HOST_SKIPS_GIGASWITCH       TRUE
 *      HOST_SKIPS_LANE             FALSE
 *      LINK_HAS_LLC_SNAP           TRUE
 *      LINK_HAS_NLPID              don't care
 *
 * (6) ATM Forum LAN emulation encapsulation followed by LLC/SNAP
 *      HOST_SKIPS_GIGASWITCH       FALSE
 *      HOST_SKIPS_LANE             TRUE
 *      LINK_HAS_LLC_SNAP           TRUE
 *      LINK_HAS_NLPID              don't care
 *
 **********************************************************************
 *
 * not implemented yet, because TCP sequence number
 * would straddle cells (current data structures actually
 * cause 2 too many bytes to be declared in first cell
 * and 2 too few bytes to be declared in second cell):
 *
 * (7) DIGITAL FDDI GigaSwitch encapsulation followed by NLPID
 *      HOST_SKIPS_GIGASWITCH       TRUE
 *      HOST_SKIPS_LANE             FALSE
 *      LINK_HAS_LLC_SNAP           FALSE
 *      LINK_HAS_NLPID              TRUE
 *
 * (8) ATM Forum LAN emulation encapsulation followed by EtherType
 *      HOST_SKIPS_GIGASWITCH       FALSE
 *      HOST_SKIPS_LANE             TRUE
 *      LINK_HAS_LLC_SNAP           FALSE
 *      LINK_HAS_NLPID              TRUE
 */



/* how much difference to expect between \AAL5 length and IP length
 * and how much of TCP payload is in second cell beyond 8th byte
 */
#if HOST_SKIPS_GIGASWITCH || HOST_SKIPS_LANE
#   if LINK_HAS_LLC_SNAP
#       define MAC_TO_IP_OFFSET        24
#   else
#       if LINK_HAS_NLPID
#           define MAC_TO_IP_OFFSET    18
#       else
#           define MAC_TO_IP_OFFSET    16
#       endif
#   endif
#else
#   if LINK_HAS_LLC_SNAP
#       define MAC_TO_IP_OFFSET        8
#   else
#       if LINK_HAS_NLPID
#           define MAC_TO_IP_OFFSET    2
#       else
#           define MAC_TO_IP_OFFSET    0
#       endif
#   endif
#endif
#define PAY9UP_SIZE                    (48 - MAC_TO_IP_OFFSET)



/* first cell of a packet
 */
struct tracerec_2;
struct tracerec
{
    CardClock when;

    union {
        AtmHeaderCoerce atm;
        tracerec_2 *next;
    };

#if CARD_COPIES_ATM_PAYLOAD

#if HOST_SKIPS_GIGASWITCH
    struct {
        u_char unknown[ 4 ];
        u_char fddi_src[ 6 ];     /* big-endian, bits are swapped in each byte */
        u_char fddi_dst[ 6 ];     /* big-endian, bits are swapped in each byte */
    } gigaswitch;
#else
#if HOST_SKIPS_LANE
    // see ftp://ftp.atmforum.com/pub/approved-specs/af-lane-0021.000.pdf
    // tables 7, 8, 23 on pages 49, 50, 92 respectively
    //
    // we assume IEEE 802.3 here
    //
    struct {
        u_short hdr;              /* big-endian, 0xFF00 if control, 0 or LE Client ID if data */
        u_char dst[ 6 ];          /* big-endian */
        u_char src[ 6 ];          /* big-endian */
        u_short type_or_length;   /* big-endian, >=1536 is ethertype (must get length from AAL5 trailer), */
                                  /*   >0 is length (can identify padding) and LLC follows, */
                                  /*   0 neither length nor ethertype provided and LLC follows */
    } lane;
#endif
#endif

#if LINK_HAS_LLC_SNAP
    struct {
        union {
            struct {
                u_char dsap;      /* (destination services access point) */
                u_char ssap;      /* (source services access point) */
            } lsap_pieces;

            u_short lsap;         /* (link services access point) */
#define LSAP_SNAP     0xAAAA
        };
        u_char ctl;               /* (control) */
#define CTL_SNAP      0x03
    } tr_llc;                     /* 0xAA 0xAA 0x03 (SNAP header follows) */

    struct {
        u_char oui[ 3 ];          /* 0x00 0x00 0x00 (org. unique ID) */
        u_short ethertype;        /* 0x08 0x00 (IP) */
#define ETHERTYPE_IP  0x0800
#define ETHERTYPE_ARP 0x0803      /* anything that isn't IP is fine here */
#if HOST_IS_BIG_ENDIAN
#define ETHERTYPE_IP_SWAPPED    ETHERTYPE_IP
#define ETHERTYPE_ARP_SWAPPED   ETHERTYPE_ARP
#else
#define ETHERTYPE_IP_SWAPPED    0x0008
#define ETHERTYPE_ARP_SWAPPED   0x0308
#endif
    } tr_snap;
#else       // !LINK_HAS_LLC_SNAP

#if LINK_HAS_NLPID
    union {
        struct {
            u_char ctl;
#define CTL_SNAP      0x03

            u_char mux;         // I made this up: is there correct name?
#define MUX_CLNS      0x66      // wrong: fill in value when I know it
#define MUX_ARP       0x77      // wrong: fill in value when I know it
#define MUX_IP        0xCC      // verified from link traces
        } tr_nlpid;

        u_short ethertype;
#define ETHERTYPE_IP            ((CTL_SNAP << 8) | MUX_IP)
#define ETHERTYPE_CLNS          ((CTL_SNAP << 8) | MUX_CLNS)
#if HOST_IS_BIG_ENDIAN
#define ETHERTYPE_IP_SWAPPED    ETHERTYPE_IP
#define ETHERTYPE_CLNS_SWAPPED  ETHERTYPE_CLNS
#else
#define ETHERTYPE_IP_SWAPPED    ((MUX_IP << 8) | CTL_SNAP)
#define ETHERTYPE_CLNS_SWAPPED  ((MUX_CLNS << 8) | CTL_SNAP)
#endif
    };
#endif      // LINK_HAS_NLPID
#endif      // !LINK_HAS_LLC_SNAP

    struct ip tr_ip;

    /* IP options would go here */

    struct tcphdr_1 tr_tcp1;         /* ports */

#if (!HOST_SKIPS_GIGASWITCH && !HOST_SKIPS_LANE) || !LINK_HAS_LLC_SNAP
    struct tcphdr_2 tr_tcp2;         /* seq and ack */
#endif
#if !HOST_SKIPS_GIGASWITCH && !HOST_SKIPS_LANE
    struct tcphdr_3 tr_tcp3;         /* everything but ports,seq,ack */

#if !LINK_HAS_LLC_SNAP
    /* TCP options would go here */

    u_char tr_tcp_pay1to6[ 6 ];
#if !LINK_HAS_NLPID
    u_char tr_tcp_pay7to8[ 2 ];
#endif      // !LINK_HAS_NLPID
#endif      // !LINK_HAS_LLC_SNAP
#endif      // !HOST_SKIPS_GIGASWITCH && !HOST_SKIPS_LANE
#endif      // CARD_COPIES_ATM_PAYLOAD
};



/* second cell of a packet
 */
struct tracerec_2
{
    CardClock when;

    union {
        AtmHeaderCoerce atm;
        tracerec_2 *next;
    };

#if CARD_COPIES_ATM_PAYLOAD

#if (HOST_SKIPS_GIGASWITCH || HOST_SKIPS_LANE) && LINK_HAS_LLC_SNAP
    struct tcphdr_2 tr_tcp2;         /* seq and ack */
#endif

#if HOST_SKIPS_GIGASWITCH || HOST_SKIPS_LANE
    struct tcphdr_3 tr_tcp3;         /* everything but ports,seq,ack */
#endif

#if LINK_HAS_LLC_SNAP || HOST_SKIPS_GIGASWITCH || HOST_SKIPS_LANE
    /* TCP options would go here */

    Bit8 tr_tcp_pay1to6[ 6 ];        /* 1st to 6th payload bytes in a TCP packet */
#endif
#if LINK_HAS_LLC_SNAP || LINK_HAS_NLPID || HOST_SKIPS_GIGASWITCH || HOST_SKIPS_LANE
    Bit8 tr_tcp_pay7to8[ 2 ];        /* 7th to 8th payload bytes in a TCP packet */
#endif
    Bit8 tr_tcp_pay9up[ PAY9UP_SIZE ];   /* 9th+ payload bytes in a TCP packet */

#endif      // CARD_COPIES_ATM_PAYLOAD
};



/* last cell of an AAL5 packet
 */
struct tracerec_last
{
    CardClock when;

    union {
        AtmHeaderCoerce atm;
        tracerec_2 *next;
    };

#if CARD_COPIES_ATM_PAYLOAD
    Bit8 padding[ 40 ];

    Bit16 control;
    Bit16 length;
    Bit32 crc;
#endif      // CARD_COPIES_ATM_PAYLOAD
};



/* repeat count and cell to be sent
 */
struct tracerec_send
{
    Bit32 count;                // little-endian order

    AtmHeaderCoerce atm;        // big-endian order, regardless of card

    Bit8 padding[ 40 ];         // actually 48 if not last cell

    Bit16 control;
    Bit16 length;
    Bit32 crc;
};



#if sizeof(struct tracerec) != sizeof(struct tracerec_last)
#   error tracerec not same size as tracerec_last
#   wishicouldprint GigaSwitch encapsulation can be combined with LLC/SNAP but not NLPID
#endif
#if sizeof(struct tracerec_2) != sizeof(struct tracerec_last)
#   error tracerec_2 not same size as tracerec_last
#   wishicouldprint GigaSwitch encapsulation can be combined with LLC/SNAP but not NLPID
#endif



// everybody should use these #defines to get to fields
// inside cellhashrec instead of accessing fields themselves
// so I can pull the rug out from under them by relocating
// fields, and they won't have to change to work afterwards
//
// conversely, if they need a field not here, it is my duty
// to add it here
//
#define tr_when        first->when
#define tr_gotlast     last->atm.bits.LAST              // did we capture last cell of packet?
#define tr_atm         last->atm
#if LINK_HAS_LLC_SNAP
#   define tr_lsap     first->tr_llc.lsap
#   define tr_ctl      first->tr_llc.ctl
#   define tr_oui      first->tr_snap.oui
#   define tr_etype    first->tr_snap.ethertype
#else
#if LINK_HAS_NLPID
#   define tr_ctl      first->tr_nlpid.ctl
#   define tr_mux      first->tr_nlpid.mux
#   define tr_etype    first->ethertype
#endif
#endif
#define tr_iphl        first->tr_ip.ip_hl
#define tr_tos         first->tr_ip.ip_tos
#define tr_len         first->tr_ip.ip_len
#define tr_offset      first->tr_ip.ip_off
#define tr_ident       first->tr_ip.ip_id
#define tr_proto       first->tr_ip.ip_p
#define tr_src         first->tr_ip.ip_src.s_addr
#define tr_dst         first->tr_ip.ip_dst.s_addr
#define tr_sport       first->tr_tcp1.th_sport
#define tr_dport       first->tr_tcp1.th_dport
#if (HOST_SKIPS_GIGASWITCH || HOST_SKIPS_LANE) && LINK_HAS_LLC_SNAP
#   define tr_seq      second->tr_tcp2.th_seq
#   define tr_ack      second->tr_tcp2.th_ack
#else
#   define tr_seq      first->tr_tcp2.th_seq
#   define tr_ack      first->tr_tcp2.th_ack
#endif
#if HOST_SKIPS_GIGASWITCH || HOST_SKIPS_LANE
#   define tr_tcplen   second->tr_tcp3.th_off
#   define tr_tcpflags second->tr_tcp3.th_flags
#   define tr_win      second->tr_tcp3.th_win
#else
#   define tr_tcplen   first->tr_tcp3.th_off
#   define tr_tcpflags first->tr_tcp3.th_flags
#   define tr_win      first->tr_tcp3.th_win
#endif
#if LINK_HAS_LLC_SNAP || HOST_SKIPS_GIGASWITCH || HOST_SKIPS_LANE
#   define tr_pay1to6       second->tr_tcp_pay1to6
#   define tr_gotpay1to6    second!=NULL
#else
#   define tr_pay1to6       first->tr_tcp_pay1to6
#   define tr_gotpay1to6    first!=NULL
#endif
#if LINK_HAS_NLPID || LINK_HAS_LLC_SNAP || HOST_SKIPS_GIGASWITCH || HOST_SKIPS_LANE
#   define tr_pay7to8       second->tr_tcp_pay7to8
#   define tr_gotpay7to8    second!=NULL
#else
#   define tr_pay7to8       first->tr_tcp_pay7to8
#   define tr_gotpay7to8    first!=NULL
#endif
#define tr_pay9up      second->tr_tcp_pay9up
#define tr_gotpay9up   second!=NULL
#define tr_maclen      last->length
#define tr_macctl      last->control
#define tr_maccrc      last->crc



/* how received cells are stored on disk (always big-endian byte ordering)
 */
struct traceblock
{
    // how many of buffers in this block are filled in?
    //
    // in big-endian order on disk
    //
    Bit32 used;

    // which card filled in this block?
    //

    // PCI bus handle of the card
    //
    // in big-endian order on disk
    //
    // also, 7 or below here and version==0 means an old-format CAIDA trace
    //
    Bit16 cardnum;

    // global array index of the card
    //
    Bit8 cardindex;

    Bit8 card_details_are_filled_in;

    // 0 is what MCI or CAIDA had before, 1 is this format
    //
    Bit32 version;

    // how many cells were lost by card during this block?
    //
    Bit32 lost;

    // fields below here may only be filled in once per card per file
    //

    // how many buffers were written to disk with this block header?
    // (possibly larger than used)
    //
    Bit32 max_used;

    // how to get an host-absolute-time / card-relative-time pair:
    //
    // on the FORE OC3 card, the card interrupts the host before
    // writing the first cell of a new block (but not the first
    // block ever), so record the entry time of the interrupt
    // handler and the timestamp of the first cell of the next block
    //
    // on the Applied Telecom OC12 card, the card interrupts the host
    // after writing the last cell of each block, so record the entry
    // time of the interrupt handle and the timestamp of the last
    // cell of the just-filled block
    //
    Bit32 host_time_seconds;        // since 00:00:00.0 GMT Jan 1 1970
    Bit32 host_time_nanoseconds;
    Bit32 card_time_high;           // in same format as in data of block
    Bit32 card_time_low;

    //    149,760 (actually    149,759.980) for OC3c
    //    599,040 (actually    599,039.920) for OC12c
    //  2,396,160 (actually  2,396,159.680) for OC48c
    //  9,584,639 (actually  9,584,638.720) for OC192c
    // 38,338,555 (actually 38,338,554.880) for OC768c
    //
    Bit32 kilobits_per_second;

    // 0 FORE PCA200e OC3 card's 64 bits total (listed from msb to lsb)
    //      32 bit software enhancement of...
    //      16 bit hardware ticks
    //      14 bit PCI bus FIFO depth
    //      2  bit cell count
    //
    // 1 Applied Telecom POINT OC12c card's 64 bits total (listed from
    //      56 bits hardware counter
    //      6 bits unused
    //      2 bits cell count
    //
    // 2-255 unassigned as yet
    //
    Bit8 timestamp_version;

    Bit8 timestamp_is_little_endian;

    Bit8 atm_header_is_little_endian;

    Bit8 oam_rm_cells_were_copied;

    // -1 is raw trace
    //
    Bit16 normal_cells_per_packet;

    Bit16 padding1;                 // gets me up to a 32-bit boundary

    Bit32 timestamp_ticks_per_second;

    // background: not all of the bits of VPI and VCI can be used
    // by the card to look up reassembly state, and cells that don't
    // match a host-settable value in those "other" bits are dropped
    // and counted
    //
    Bit32 cells_dropped_for_unexpected_vpi_vci;

    // must fill up a disk sector, otherwise write is 40 times slower!
    //
    Bit8 padding2[ (SECTOR_SIZE / 2) - 14 * sizeof(Bit32) ];

    // probably holds site name, which link was monitored, etc.
    //
    Bit8 name[ SECTOR_SIZE / 2 ];

    // maybe not all these are used
    //
    struct tracerec data[ COMPL_RING_ENTRIES ];
};



/* how cells to transmit are stored on disk (always big-endian byte ordering)
 */
struct traceblock_send
{
    // how many of buffers in this block are filled in?
    //
    // in big-endian order on disk
    //
    Bit32 used;

    // we don't know which cardnum will send this block
    //

    // global array index of the card we want to send the block
    //
    Bit8 cardindex;

    // must fill up a disk sector, otherwise write is 40 times slower!
    //
    Bit8 padding[ SECTOR_SIZE - sizeof(Bit8) - sizeof(Bit32) ];

    // maybe not all these are used
    //
    struct tracerec_send data[ SEND_RING_ENTRIES ];
};



/* lookup table to mirror-swap the bits of a byte,
 * mostly for FDDI MAC addresses
 *
 * (see INTEGRAT\TRACE.C for initialization values)
 */
extern Bit8 bitswap[];



/* some useful functions, also defined in INTEGRAT\TRACE.C
 */
extern void dump_cell( void *sock, struct tracerec *cell, const char *indent="" );



#endif      /* not included yet */
