#ifndef _FPU_H
#define _FPU_H



#include "..\types\types.h"



typedef struct
{
    Bit16 invalid_operation_mask : 1;       // IM
    Bit16 denormalized_operand_mask  : 1;   // DM
    Bit16 zero_divide_mask      : 1;        // ZM
    Bit16 overflow_mask         : 1;        // OM
    Bit16 underflow_mask        : 1;        // UM
    Bit16 precision_mask        : 1;        // PM
    Bit16                       : 2;        // reserved
    Bit16 precision_control     : 2;        // PC
    Bit16 rounding_control      : 2;        // RC
    Bit16 infinity_control      : 1;        // 1=affine 0=projective    387+ always affine (-inf < +inf)
    Bit16                       : 4;        // reserved

} FpuControl;

typedef enum FpuControlRounding
{
    round_down_to_nearest_or_even = 0,
    round_down_toward_infinity,
    round_up_toward_infinity,
    chop_toward_zero
};

typedef enum FpuControlPrecision
{
    single_precision_24_bit = 0,
    reserved_precision,
    double_precision_53_bit,
    extended_precision_64_bit
};



typedef struct
{
    // exception bits
    //
    Bit16 invalid_operation_exception : 1;  // IE
    Bit16 denormalized_operand_exception : 1; // DE
    Bit16 zero_divide_exception : 1;        // ZE
    Bit16 overflow_exception    : 1;        // OE       arithmetic
    Bit16 underflow_exception   : 1;        // UE       arithmetic
    Bit16 precision_exception   : 1;        // PE
    Bit16 stack_fault_exception : 1;        // SF
    Bit16 error_summary         : 1;        // ES       are any unmasked exception bits set?

    // see below for condition code meanings
    //
    Bit16 condition_code_0      : 1;        // C0       SAHF copies to integer carry flag
    Bit16 condition_code_1      : 1;        // C1
    Bit16 condition_code_2      : 1;        // C2       SAHF copies to integer parity flag
    Bit16 top_of_stack          : 3;        // TOP      which register is top of stack?
    Bit16 condition_code_3      : 1;        // C3       SAHF copies to integar zero flag
    Bit16 busy                  : 1;        // B

} FpuStatus;

// condition code interpretation
//
// instruction              C0          C3          C2              C1
//--------------------------------------------------------------------------
// FCOM FCOMP FCOMPP        result of comparison    operand         zero or
// FTST FUCOMPP FICOM                               not             O/U#
// FICOMP                                           compatible
//--------------------------------------------------------------------------
// FXAM                     operand class .....................     sign or
//                                                                  O/U#
//--------------------------------------------------------------------------
// FPREM FPREM1             Q2          Q1          reduction       Q0
//                                                  0=complete      or
//                                                  1=incomplete    O/U#
//                                                                  or O/U#
//--------------------------------------------------------------------------
// FIST FBSTP FRINDINT      UNDEFINED   UNDEFINED   UNDEFINED       roundup
// FST FSTP FADD FMUL                                               or
// FDIV FDIVR FSUB FSUBR                                            O/U#
// FSCALE FSQRT FPATAN
// F2XM1 FYL2X FYL2XP1
//--------------------------------------------------------------------------
// FPTAN FSIN FCOS          UNDEFINED   UNDEFINED   reduction       roundup
// FSINCOS                                          0=complete      or O/U#
//                                                  1=incomplete    (UNDEFINED
//                                                                  if C2=1)
//--------------------------------------------------------------------------
// FCHS FABS FXCH           UNDEFINED   UNDEFINED   UNDEFINED       zero or
// FINCSTP FDECSTP                                                  O/U#
// constant loads
// FXTRACT FLD FILD
// FBLD FSTP (ext real)
//--------------------------------------------------------------------------
// FLDENC FRSTOR            each bit loaded from memory ..................
//--------------------------------------------------------------------------
// FLDCW FSTENV             UNDEFINED   UNDEFINED   UNDEFINED       UNDEFINED
// FSTCW FSTSW
// FCLEX
//--------------------------------------------------------------------------
// FINIT FSAVE              zero        zero        zero            zero
//--------------------------------------------------------------------------
//



// FNSTSW AX instruction
//
// sorry result is an integer not structure
//
#define get_fpu_status() (__emit__(0xDF, 0xE0), _AX)

// FNCLEX instruction, takes 9 clocks
//
#define clear_fpu_status() __emit__(0xDB, 0xE2)

#if 0
// sorry must invoke TASM
// sorry must put in memory
// sorry uses inline assembly so cannot return a value
// sorry doesn't like union members
//
#define get_fpu_control(x) asm fnstcw word ptr x
#else
#ifdef __cplusplus
extern "C"
#endif
FpuControl get_fpu_control( void );
#endif

#if 0
// sorry must invoke TASM
// sorry uses inline assembly so cannot return a value
// sorry (I predict it) doesn't like union members
//
#define set_fpu_control(x) asm fnldcw x
#else
#ifdef __cplusplus
extern "C"
#endif
void set_fpu_control( FpuControl chosen );
#endif



#endif          // not included yet
